import os
from pathlib import Path
import textwrap

IGNORE_DIRS = {".venv", "__pycache__", ".git", ".mypy_cache", ".idea", "*.egg", "*.egg-info", "node_modules", ".vscode", ".pytest_cache", "dist"}

def get_project_name(path : Path):
    return path.name if path.is_dir() else path.parent.name

def build_tree(path: Path, prefix="") -> list[str]:
    lines = []
    entries = sorted([p for p in path.iterdir() if p.name not in IGNORE_DIRS])
    for i, entry in enumerate(entries):
        connector = "└── " if i == len(entries) - 1 else "├── "
        lines.append(f"{prefix}{connector}{entry.name}")
        if entry.is_dir():
            extension = "    " if i == len(entries) - 1 else "│   "
            lines.extend(build_tree(entry, prefix + extension))
    return lines

def generate_readme(project_name, tree_structure):
    return f"""\
# {project_name}

## 📁 專案結構範例

```
{project_name}/
{tree_structure}
```

---

## 🚀 安裝與啟動說明

此專案使用 [uv](https://github.com/astral-sh/uv) 來管理 Python 環境與套件。
以下是安裝與啟動的步驟：

### 1. 安裝 uv

如果已安裝 Rust，可透過 cargo 安裝：

```bash
cargo install uv
```

如果有基本的python跟pip

```bash
pip install uv
```

或使用官方安裝腳本：

```ps
curl -LsSf https://astral.sh/uv/install.sh | sh
```

如果是在 windows底下:
```ps
powershell -ExecutionPolicy ByPass -c "irm https://astral.sh/uv/install.ps1 | iex"
```

---

### 1. 同步 `pyproject.toml` 

```bash
uv sync
```

---

### 3. 進入開發環境

```bash
source .venv/bin/activate   # 或使用 uv shell
```

### 4. 啟動專案
```bash
uv run main.py
```

"""


if __name__ == "__main__":
    import argparse
    import os
    parser = argparse.ArgumentParser(description="Generate a README.md file for the current project.")
    parser.add_argument("-p", "--project_name", type=str, help="Name of the project. Defaults to the current directory name.")
    parser.add_argument("-o", "--output"      , type=str, default="README.md", help="Output file name. Defaults to README.md.")
    parser.add_argument("-d", "--directory"   , type=str, default=os.getcwd(), help="Directory to scan. Defaults to the current working directory.")
    args = parser.parse_args()

    project_path    = Path(args.directory).resolve() if args.directory else Path.cwd()

    project_name = args.project_name if args.project_name else None
    if not project_name:
        project_name    = get_project_name(project_path)

    output_filename = args.output
    output_filepath = Path(project_path) / output_filename
    
    # 獲取專案結構
    tree_lines = build_tree(project_path)
    tree       = "\n".join(tree_lines)
    
    # 生成 README 內容
    readme_content = generate_readme(project_name, tree)
    
    with open(output_filepath, "w", encoding="utf-8") as f:
        f.write(textwrap.dedent(readme_content))
    
    print(f"README.md 已生成於 {output_filepath}")



from typing import Optional

import colorama as cm
import itertools
import asyncio
import time
import sys

from minwei_tools.dotter import piano, slash  # Importing from the dotter module

class AsyncDotter:
    def __init__(self, message: str = "Thinking", delay: float = 0.1,
                 cycle: list[str] = ["", ".", ". .", ". . ."],
                 show_timer: bool = False) -> None:

        self.spinner = itertools.cycle(cycle)
        self.message    : str                    = message
        self.delay      : float                  = delay
        self.show_timer : bool                   = show_timer
        self._task      : Optional[asyncio.Task] = None
        self._start_time: Optional[float]        = None
        self._running   : bool                   = False
        self.inserted_messages: list[str]       = []  # Store inserted messages
        self.max_inserted_line  = 0
        
    def format_elapsed(self, elapsed: float) -> str:
        if elapsed < 300:
            return f"{elapsed:.1f}s"
        else:
            mins = int(elapsed // 60)
            secs = int(elapsed % 60)
            return f"{mins}m{secs:02d}s"

    async def _dot(self):
        self._start_time = time.time()
        while self._running:
            elapsed = time.time() - self._start_time
            text = f"{self.message} {next(self.spinner)}"

            if self.show_timer:
                timer_str = f"[{self.format_elapsed(elapsed)}]"
                text = timer_str + ' ' + text

            sys.stdout.write(f"\r{text}\n")
            for msg in self.inserted_messages:
                sys.stdout.write(f"    {cm.Style.DIM}{msg}{cm.Style.RESET_ALL}\n")
            sys.stdout.flush()
            await asyncio.sleep(self.delay)
            self.max_inserted_line = max(self.max_inserted_line, len(self.inserted_messages))
            lines_to_clear = 1 + self.max_inserted_line
            for _ in range(lines_to_clear):
                sys.stdout.write("\033[F")  # Move cursor up one line
                sys.stdout.write("\033[K")  # Clear the entire line
            sys.stdout.flush()
            
    async def insert_message(self, new_message: str, max_str : int = 5, prefix : str = "=>"):
        self.inserted_messages.append(prefix + " " + new_message)
        while len(self.inserted_messages) > max_str:
            self.inserted_messages.pop(0)

    async def update_message(self, new_message: str, delay: float = 0.1):
        await asyncio.sleep(delay)
        sys.stdout.write(f"\r{' ' * (len(self.message) + 20)}\r")
        sys.stdout.flush()
        self.message = new_message

    async def __aenter__(self):
        self._running = True
        self._task = asyncio.create_task(self._dot())
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._running = False
        if self._task:
            await self._task
        sys.stdout.write(f"\r{' ' * (len(self.message) + 20)}\r")
        sys.stdout.flush()
        
        
if __name__ == "__main__":
    import asyncio

    async def main():
        async with AsyncDotter("Loading", show_timer=False, cycle=piano, delay=0.1) as d:
            
            await d.insert_message("This is a test message 1")
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 2")
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 3")
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 4")
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 5")
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 6")
            await asyncio.sleep(1)
            await d.update_message("[*] Player log grabbed", delay=0.1)
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 7")
            await asyncio.sleep(1)
            await d.insert_message("This is a test message 8")
            await asyncio.sleep(1)        
            await d.insert_message("This is a test message 9")

    asyncio.run(main())


from __future__ import annotations
from typing import Generic, Union, TypeVar, Callable

T = TypeVar('T')
E = TypeVar('E')

class Result(Generic[T, E]):
    __value : T = None
    __error : E = None
    __match_args__ = ('value',)

    @property
    def value(self) -> Union[T, E]:
        return self.__value if self.is_ok() else self.__error
    
    @property
    def error(self) -> Union[T, E]:
        return self.__error if self.is_err() else self.__value
    
    @value.setter
    def value(self, value: T) -> None:
        self.__value = value

    @error.setter
    def error(self, value: E) -> None:
        self.__error = value

    def is_ok(self) -> bool:
        return isinstance(self, Ok)
    
    def is_err(self) -> bool:
        return isinstance(self, Err)
    
    def unwrap(self) -> T:
        if self.is_ok():
            return self.value
        raise ValueError("Called unwrap on an Err value")
    
    def unwrap_err(self) -> E:
        if self.is_err():
            return self.error
        raise ValueError("Called unwrap_err on an Ok value")
    
    def map(self, func: Callable[[T], T]) -> Result[T, E]:
        if self.is_ok():
            return Ok(func(self.value))
        return self  # Return the Err unchanged

    def map_err(self, func: Callable[[E], E]) -> Result[T, E]:
        if self.is_err():
            return Err(func(self.error))
        return self  # Return the Ok unchanged

    def and_then(self, func: Callable[[T], Result[T, E]]) -> Result[T, E]:
        if self.is_ok():
            return Ok(func(self.value))
        return self  # Return the Err unchanged

    def or_else(self, func: Callable[[E], Result[T, E]]) -> Result[T, E]:
        if self.is_err():
            return Err(func(self.error))
        return self  # Return the Ok unchanged

    def __str__(self) -> str:
        if self.is_ok():
            return f"Ok({self.value})"
        return f"Err(\"{self.error}\")"

class Ok(Result, Generic[T, E]):
    def __init__(self, value: T) -> None:
        self.value = value

class Err(Result, Generic[T, E]):
    def __init__(self, error: E) -> None:
        self.error = error
